use std::any::{Any};

use dyn_clonable::*;
use macro_module_any::Module;
use speech_feedback::Text2Speech;

// Module trait
#[clonable]
pub trait Module: Any + Clone + Send + Sync {
    fn as_any(&self) -> &dyn Any;
}

// Actual modules
// SPEECH FEEDBACK MODULE
#[derive(Clone, Module)]
pub struct SpeechFeedbackModule {
    pub speech: Box<Text2Speech>
}
impl SpeechFeedbackModule {
    pub fn new() -> Self {
        SpeechFeedbackModule {
            speech: Box::new(Text2Speech::new())
        }
    }
    pub fn enqueue_for_read(&self, text: &'static str) {
        self.speech.enqueue_for_read(text);
    }
}

// FUZZY LOGIC MODULE
#[derive(Clone, Module)]
pub struct FuzzyLogicModule {
    pub engine: Box<rsfuzzy::Engine>
}
impl FuzzyLogicModule {
    pub fn new() -> Self {
        FuzzyLogicModule {
            engine: Box::new(rsfuzzy::Engine::new())
        }
    }
}
