
use csharp_codegen::*;

#[test]
fn empty_scope() {
    let scope = Scope::new();
    assert_eq!(scope.to_string(), "");
}

#[test]
fn scope_with_using() {
    let mut scope = Scope::new();
    scope.add_using("System.Linq");
    assert_eq!(scope.to_string(), "using System.Linq;\n\n");
}

#[test]
fn empty_namespace() {
    let mut scope = Scope::new();
    scope.new_namespace("TestNamespace");
    assert_eq!(scope.to_string(), "namespace TestNamespace {}\n\n");
}

#[test]
fn scope_with_struct() {
    let mut scope = Scope::new();

    let mut strct = Struct::new("TestStruct");
    strct.new_attribute("name", "string");
    scope.add_struct(strct);

    assert_eq!(scope.to_string(), "struct TestStruct {\n    public string name;\n\n}\n\n");
}

#[test]
fn scope_with_class() {
    let mut scope = Scope::new();

    let mut class = Class::new("TestClass");
    class.new_attribute("name", "string");
    scope.add_class(class);

    assert_eq!(scope.to_string(), "class TestClass {\n    public string name;\n\n}\n\n");
}

#[test]
fn namespace_with_empty_enum() {
    let mut scope = Scope::new();
    scope.new_namespace("TestNamespace").new_enum("TestEnum");
    assert_eq!(scope.to_string(), "namespace TestNamespace {\n\n    enum TestEnum {}\n\n}\n\n");
}

#[test]
fn namespace_with_enum() {
    let mut scope = Scope::new();

    let mut enm = Enum::new("TestEnum");
    enm.new_modifier("public")
        .new_modifier("static")
        .new_option("Width")
        .new_option("Height");

    scope.new_namespace("TestNamespace").add_enum(enm);

    assert_eq!(scope.to_string(), "namespace TestNamespace {\n\n    \
        public static enum TestEnum {\n        Width,\n        Height,\n    }\n\n}\n\n");
}

#[test]
fn class_with_method() {
    let mut scope = Scope::new();

    let mut method = Method::new("TestMethod", "void");
    method.new_modifier("public")
        .new_modifier("static")
        .new_parameter("width", "double")
        .new_parameter("height", "double");

    scope.new_class("TestClass").add_method(method);

    assert_eq!(scope.to_string(), "class TestClass {\n    \
        public static void TestMethod(double width, double height) {}\n\n\n}\n\n");
}

#[test]
fn class_with_method_signature() {
    let mut scope = Scope::new();

    let mut signature = MethodSignature::new("TestSignature", "void");
    signature.new_header("DllImport", "__Internal", None, false);

    let mut class = Class::new("TestClass");
    class.new_attribute("name", "string");
    class.add_method_signature(signature);

    scope.add_class(class);

    assert_eq!(scope.to_string(), "class TestClass {\n    public string name;\n\n    \
        [DllImport (__Internal)]\n    void TestSignature();\n\n\n}\n\n");
}