use std::sync::{atomic, Arc, Mutex};

pub struct Inner<K, V>
where
    K: Eq,
{
    pub data: Vec<(K, V)>,
    pub epochs: Arc<Mutex<Vec<Arc<atomic::AtomicUsize>>>>,
    ready: bool,
}

impl<K, V> Clone for Inner<K, V>
where
    K: Eq + Clone,
    V: Clone,
{
    fn clone(&self) -> Self {
        Inner {
            data: self.data.clone(),
            epochs: Arc::clone(&self.epochs),
            ready: self.ready,
        }
    }
}

impl<K, V> Inner<K, V>
where
    K: Eq,
{
    pub fn with_capacity(capacity: usize) -> Self {
        Inner {
            data: Vec::with_capacity(capacity),
            epochs: Default::default(),
            ready: false,
        }
    }

    pub fn mark_ready(&mut self) {
        self.ready = true;
    }

    pub fn is_ready(&self) -> bool {
        self.ready
    }

    pub fn register_epoch(&self, epoch: &Arc<atomic::AtomicUsize>) {
        self.epochs.lock().unwrap().push(Arc::clone(epoch));
    }
}
