mod inner;
use inner::Inner;

#[derive(Clone)]
enum Operation<K, V> {
    Replace(K, V),
    Push(K, V),
    Remove(K, V),
    Clear(K),
}

mod write;
pub use write::WriteHandle;

mod read;
pub use read::ReadHandle;

// Options for how to initialize the vector.
pub struct Options
{
    capacity: Option<usize>,
}

impl Default for Options {
    fn default() -> Self {
        Options {
            capacity: None,
        }
    }
}

impl Options
{
    // Set the initial capacity for the vector.
    pub fn with_capacity(self, capacity: usize) -> Options {
        Options {
            capacity: Some(capacity),
        }
    }

    // Create the vector, and construct the read and write handles used to access it.
    pub fn construct<K, V>(self) -> (ReadHandle<K, V>, WriteHandle<K, V>)
    where
        K: Eq + Clone,
        V: Clone,
    {
        let inner = if let Some(cap) = self.capacity {
            Inner::with_capacity(cap)
        } else {
            Inner::with_capacity(0)
        };

        let mut w_handle = inner.clone();
        w_handle.mark_ready();
        let r = read::new(inner);
        let w = write::new(w_handle, r.clone());
        (r, w)
    }
}

// Create an empty eventually consistent vector.
pub fn new<K, V>() -> (
    ReadHandle<K, V>,
    WriteHandle<K, V>,
)
where
    K: Eq + Clone,
    V: Clone,
{
    Options::default().construct()
}
