use std::env;
use std::fs;
use std::path::Path;

use ontology_parser::{generate_types, generate_api, parse};

const ONTOLOGY_FILE_PATH: &str = "../ontology.owl";
const KNOWLEDGE_GRAPH_TYPES_FILE: &str = "knowledge_graph_types.rs";
const RUST_API_FOLDER: &str = "rust_api";

const UNITY_API_FOLDER: &str = "unity_api";
const UNITY_API_FILE: &str = "KnowledgeGraphApi.cs";


fn main() {
    // Parse ontology
    let parsed_ontology = parse(ONTOLOGY_FILE_PATH);

    // Generate code from ontology file
    // Rust knowledge graph types
    let current_directory = env::current_dir().unwrap();
    let output_directory = env::var_os("OUT_DIR").unwrap();
    let rust_types_file_path = Path::new(&output_directory).join(KNOWLEDGE_GRAPH_TYPES_FILE);

    let rust_api_directory = current_directory.join(RUST_API_FOLDER);
    fs::create_dir_all(&rust_api_directory).unwrap();
    let rust_types_file_path2 = Path::new(&rust_api_directory).join(KNOWLEDGE_GRAPH_TYPES_FILE);

    let generated_code = generate_types(parsed_ontology.0.clone(),
                                        parsed_ontology.1.clone());
    fs::write(
        &rust_types_file_path,
        generated_code.clone()
    ).unwrap();

    fs::write(
        &rust_types_file_path2,
        generated_code.clone()
    ).unwrap();


    // Generate unity api from ontology file
    let api_directory = current_directory.join(UNITY_API_FOLDER);
    fs::create_dir_all(&api_directory).unwrap();
    let api_file = api_directory.join(UNITY_API_FILE);

    let generated_csharp = generate_api(parsed_ontology.0.clone(),
                                        parsed_ontology.1.clone());

    fs::write(
        &api_file,
        generated_csharp.1
    ).unwrap();

    println!("cargo:rerun-if-changed=build.rs");
}