using System;
using System.Runtime.InteropServices;

namespace AssessmentSystemApi {

    public enum KnowledgeGraphNode {
        Empty = 0,
        Car = 1,
        Crosswalk = 2,
        DynamicObject = 3,
        Ego = 4,
        Intersection = 5,
        Lane = 6,
        NoEntrySign = 7,
        Pedestrian = 8,
        PriorityRoadSign = 9,
        Road = 10,
        RoadSign = 11,
        SpeedLimitSign = 12,
        StaticObject = 13,
        StopSign = 14,
        TestClass1 = 15,
        TestClass2 = 16,
        YieldSign = 17,
    }

    public enum KnowledgeGraphRelation {
        Empty = 0,
        belongsTo = 1,
        connectsTo = 2,
        contains = 3,
        has = 4,
        isAt = 5,
        isBehind = 6,
        isIn = 7,
        isInFront = 8,
        isOn = 9,
        isOnLeft = 10,
        isOnRight = 11,
        isPartOf = 12,
        passesOver = 13,
        testRelation = 14,
        walksOn = 15,
        yieldsToCarIn = 16,
    }

    [StructLayout (LayoutKind.Sequential)]
    public struct TypeIndex {
        public static uint OfGraphNode(KnowledgeGraphNode node)
        {
            var type_index = (int)node;
            return (uint)type_index;
        }

        public static uint OfGraphRelation(KnowledgeGraphRelation relation)
        {
            var type_index = (int)relation;
            return (uint)type_index;
        }


    }

    [StructLayout (LayoutKind.Sequential)]
    public struct String2Bytes {
        public static byte[] Pack(string value)
        {
            var bytes = System.Text.Encoding.UTF8.GetBytes(value);
            var result = new byte[128];
            bytes.CopyTo(result, 0);
            return result;
        }


    }

    [StructLayout (LayoutKind.Sequential)]
    public struct Vector3f {
        public float x;
        public float y;
        public float z;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct Vector2f {
        public float x;
        public float y;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct Car {
        public Vector3f carDirection;
        public int carId;
        public Vector3f carPosition;
        public float carSpeed;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct Crosswalk {
        public int crosswalkId;
        public int crosswalkPedestrianCount;
        public Vector3f crosswalkPosition;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct DynamicObject {}

    [StructLayout (LayoutKind.Sequential)]
    public struct Ego {}

    [StructLayout (LayoutKind.Sequential)]
    public struct Intersection {
        public int intersectionId;
        public Vector3f intersectionPosition;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct Lane {
        public int laneId;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
        public byte[] laneName;
        public int laneSpeedLimit;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct NoEntrySign {}

    [StructLayout (LayoutKind.Sequential)]
    public struct Pedestrian {
        public Vector2f pedestrianDirection;
        public Vector2f pedestrianPosition;
        public float pedestrianSpeed;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct PriorityRoadSign {}

    [StructLayout (LayoutKind.Sequential)]
    public struct Road {
        public int roadId;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 128)]
        public byte[] roadName;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct RoadSign {}

    [StructLayout (LayoutKind.Sequential)]
    public struct SpeedLimitSign {
        public int signSpeedLimit;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct StaticObject {}

    [StructLayout (LayoutKind.Sequential)]
    public struct StopSign {}

    [StructLayout (LayoutKind.Sequential)]
    public struct TestClass1 {
        public int testProperty1;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct TestClass2 {
        public int testProperty2;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct YieldSign {}

    [StructLayout (LayoutKind.Sequential)]
    public struct belongsTo {}

    [StructLayout (LayoutKind.Sequential)]
    public struct connectsTo {}

    [StructLayout (LayoutKind.Sequential)]
    public struct contains {}

    [StructLayout (LayoutKind.Sequential)]
    public struct has {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isAt {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isBehind {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isIn {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isInFront {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isOn {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isOnLeft {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isOnRight {}

    [StructLayout (LayoutKind.Sequential)]
    public struct isPartOf {}

    [StructLayout (LayoutKind.Sequential)]
    public struct passesOver {}

    [StructLayout (LayoutKind.Sequential)]
    public struct testRelation {
        public float testProperty;

    }

    [StructLayout (LayoutKind.Sequential)]
    public struct walksOn {}

    [StructLayout (LayoutKind.Sequential)]
    public struct yieldsToCarIn {
        public int ifDirection;

    }

    class KnowledgeGraph {
        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_car_node(ulong timestamp, uint instance_id, Car car, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_car_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_crosswalk_node(ulong timestamp, uint instance_id, Crosswalk crosswalk, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_crosswalk_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_dynamicobject_node(ulong timestamp, uint instance_id, DynamicObject dynamicobject, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_dynamicobject_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_ego_node(ulong timestamp, uint instance_id, Ego ego, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_ego_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_intersection_node(ulong timestamp, uint instance_id, Intersection intersection, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_intersection_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_lane_node(ulong timestamp, uint instance_id, Lane lane, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_lane_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_noentrysign_node(ulong timestamp, uint instance_id, NoEntrySign noentrysign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_noentrysign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_pedestrian_node(ulong timestamp, uint instance_id, Pedestrian pedestrian, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_pedestrian_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_priorityroadsign_node(ulong timestamp, uint instance_id, PriorityRoadSign priorityroadsign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_priorityroadsign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_road_node(ulong timestamp, uint instance_id, Road road, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_road_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_roadsign_node(ulong timestamp, uint instance_id, RoadSign roadsign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_roadsign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_speedlimitsign_node(ulong timestamp, uint instance_id, SpeedLimitSign speedlimitsign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_speedlimitsign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_staticobject_node(ulong timestamp, uint instance_id, StaticObject staticobject, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_staticobject_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_stopsign_node(ulong timestamp, uint instance_id, StopSign stopsign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_stopsign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_testclass1_node(ulong timestamp, uint instance_id, TestClass1 testclass1, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_testclass1_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_testclass2_node(ulong timestamp, uint instance_id, TestClass2 testclass2, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_testclass2_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_yieldsign_node(ulong timestamp, uint instance_id, YieldSign yieldsign, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_yieldsign_node(ulong timestamp, uint type_index, uint instance_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_belongsto_relation(ulong timestamp, uint instance_id, belongsTo belongsto, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_belongsto_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_connectsto_relation(ulong timestamp, uint instance_id, connectsTo connectsto, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_connectsto_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_contains_relation(ulong timestamp, uint instance_id, contains contains, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_contains_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_has_relation(ulong timestamp, uint instance_id, has has, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_has_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isat_relation(ulong timestamp, uint instance_id, isAt isat, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isat_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isbehind_relation(ulong timestamp, uint instance_id, isBehind isbehind, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isbehind_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isin_relation(ulong timestamp, uint instance_id, isIn isin, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isin_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isinfront_relation(ulong timestamp, uint instance_id, isInFront isinfront, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isinfront_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_ison_relation(ulong timestamp, uint instance_id, isOn ison, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_ison_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isonleft_relation(ulong timestamp, uint instance_id, isOnLeft isonleft, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isonleft_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_isonright_relation(ulong timestamp, uint instance_id, isOnRight isonright, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_isonright_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_ispartof_relation(ulong timestamp, uint instance_id, isPartOf ispartof, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_ispartof_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_passesover_relation(ulong timestamp, uint instance_id, passesOver passesover, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_passesover_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_testrelation_relation(ulong timestamp, uint instance_id, testRelation testrelation, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_testrelation_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_walkson_relation(ulong timestamp, uint instance_id, walksOn walkson, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_walkson_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern uint add_yieldstocarin_relation(ulong timestamp, uint instance_id, yieldsToCarIn yieldstocarin, uint from_node_type, uint from_node_index_id, uint to_node_type, uint to_node_index_id, IntPtr memory_ptr);

        [DllImport ("unity_interface", CallingConvention = CallingConvention.Cdecl)]
        public static extern bool remove_yieldstocarin_relation(ulong timestamp, uint type_index, uint index_id, IntPtr memory_ptr);


    }

}

