use knowledge_graph::graph::*;
use knowledge_graph::graph::types::*;
use interval_algebra::Interval;
use pattern_query_engine::{Pattern, Node, Relation};

#[test]
fn basic_pattern_query() {
    let mut graph = KnowledgeGraph::new_boxed();

    let class1_1 = KnowledgeGraphNode::TestClass1(TestClass1 { testProperty1: 10 });
    let class1_2 = KnowledgeGraphNode::TestClass1(TestClass1 { testProperty1: 25 });
    let class1_3 = KnowledgeGraphNode::TestClass1(TestClass1 { testProperty1: 42 });

    let class2_1 = KnowledgeGraphNode::TestClass2(TestClass2 { testProperty2: 88 });

    let relation1 = KnowledgeGraphRelation::testRelation(testRelation { testProperty: 12.0 });
    let _relation2 = KnowledgeGraphRelation::testRelation(testRelation { testProperty: 22.0 });

    let id1_1 = graph.add_node(1, 0,class1_1);
    let _id1_2 = graph.add_node(10, 0,class1_2);
    let _id1_3 = graph.add_node(15, 0,class1_3);
    let id2_1 = graph.add_node(16, 0,class2_1);
    let _id_rel1 = graph.add_relation(20, 0,
                                     relation1, id1_1, id2_1);

    let mut pattern = Pattern::new();
    pattern.node(Node::new(class1_1.type_index()))
        .with_outgoing_relation(Relation::new(relation1.type_index()))
        .to_node(Node::new(class2_1.type_index()));

    let result = pattern.find(Interval::new(0, 20), &graph);
    let _result_data = pattern.get_result();

    assert_eq!(result, true);
}