// Constants
// Bits from car1
pub const BFC1_LEFT1: u32 = 0x1;	// Handle, left blink, first step
pub const BFC1_LEFT2 : u32 = 0x2; // Handle, left blink, second step
pub const BFC1_RIGHT1: u32 = 0x4; // Handle, right blink, first step
pub const BFC1_RIGHT2: u32 = 0x8; // Handle, right blink, second step

pub const BFC1_BC: u32 = 0x10; // Handle, pushing BC button
pub const BFC1_HANDLE_BACK: u32 = 0x20; // Handle, pushing backwards you
pub const BFC1_HANDLE_FORWARD: u32 = 0x40; // Handle, pushing forward you
pub const BFC1_LEFT_DOOR_OPEN: u32 = 0x80; // Bit set when door is open
pub const BFC1_RIGHT_DOOR_OPEN: u32 = 0x100; // Bit set when door is open
pub const BFC1_SEATBELT_OFF: u32 = 0x200; // Bit set when seatbelt is off

pub const BFC1_LIGHTS_SWITCH_OFF: u32 = 0x400; // Bit set when light switch off
pub const BFC1_LIGHTS_SWITCH_PARK: u32 = 0x800; // Bit set when light switch off
pub const BFC1_LIGHTS_SWITCH_ON: u32 = 0x1000; // Bit set when light switch on
pub const BFC1_LIGHTS_BEAM_ON: u32 = 0x2000; // Bit set when beam is on

pub const BFC1_BLINKING_LEFT: u32 = 0x4000; // Bit set when blinking left.
// This bit will not flip in synchrony with the blinker, but remain on as long as we're blinking.
pub const BFC1_BLINKING_RIGHT: u32 = 0x8000; // Bit set when blinking right.
// This bit will not flip in synchrony with the blinker, but remain on as long as we're blinking.
pub const BFC1_READY_TO_START: u32 = 0x10000; // Bit set when car reports that it is ready
// to start the engine. This means that the clutch is pressed down or that the gear is in park


// Event types
// Value 0 is not valid. Used to signal that no more events is available
#[derive(Debug)]
#[repr(u32)]
pub enum EventType
{
    NoEvents = 0,
    SignSpeedLimit = 1,			// Speed limit at position. The variable "flag" not used. The variable "data" is used to give the speed limit
    SpeedScore = 2,					// Score event at a given position. The variable "flag" not used. The variable "data" is used as score
    SignRightOfWay = 3,				// Sign event. Parameter "flag" and "data" not usedconst unsigned int SignYeild = 4;       // Sign event. Parameter "flag" and "data" not used
    SignYeild = 4,					// Sign event. Parameter "flag" and "data" not used
    CommandedTrafficCarSpeed = 5,	// This is the speed a traffic car is commanded to adjust to after this point. The variable "data" contains the speed
    EyeSemantic = 6,				// Event for detected eye position.Parameter flag contains the enum of detected positions. Parameter "data" not used#pragma once
    RunControl = 7,					// Event for external control of runlevel and exercise. Only first 8 bit of flag that is used for runlevel. The rest is reseved and should be zero
    // Flag 1, data not used: Go to running run level
    // Flag 2, data not used: Go to stopping run level
    // Flag 3, data is in seconds. Restart from time relative to data value. Data should be negative
    // Flag 4, data not used: Kill Unity on all channels
    RandomNumber = 8,				// Event for same random number on all channels. Use 'Flag' or 'x, y or z' as number
    GenericEvent = 9,				// Generic event to use just to pass an event to do something across all channels.
    CarDebugBall = 10,				// Event used for debugging. Sending this creates a ball on top of a car with a given color. This can be used to debug dynamic properties for the car
    // Flag bit 0  1 = blue
    //      bit 1  1 = green
    //      bit 2  1 = red
    // Combination of bits gives 8 colors at all
    // Data: car index
    // Position: not used

    AssessmentDistanceMargin = 11,  // Event used by the assessment system to signal a change in margin for the car in front
    // flag 0 = idle following
    // flag 1 = ok distance
    // flag 2 = medium distance
    // flag 3 = short distance
    // flag 4 = critical short distance
    // Data: distance to car
    // Position of ego when the event happened

    GKScore = 12,					// Event used to manage score to other channels.
    // Flags indicate type
    // flag 0 = Kollisjon
    // flag 1 = P�kj�rsel
    // flag 2 = Vikeplikt
    // flag 3 = Sperrelinje
    // flag 4 = R�dt Lys
    // flag 5 = Feltvalg
    // flag 6 = Kj�reretning
    // flag 7 = Skiltvalg
    // flag 8 = none
    // flag 9 = Hastighet
    // flag 99 = DISPLAY SCORE

    // x = score amount / final score

    // Data not used.

    EnviroEvent = 13,				// Event used to change EnviroWeather and Time in runtime.
    // Flags indicate weather setting (index of weatherpreset).
    // x (0/1) Simulated Time (False/True)
    // y (1/24) Time of Day in Hours
    // z Day / Night Length in Minutes (Only set if x = 1).
    // Data for thunder (0 = nothing, 1 = thunderstrike).

    ClutchBeforeBrake = 14,			// Clutch was pressed before brake. Data: The cars speed in km/h when this was detected. Flag: Not used. Position of the car on detection.
    HardBrake = 15,					// Hard stop or hard deceleration.
    // flag 0 = Hard stop
    // flag 1 = Hard deceleration
    // Data: If flag==1, holds the cars avg acceleration the last 100ms, which will be a negative number (smaller is worse). Position of the car on detection.
    UnevenAcceleration = 16,		// Acceleration was uneven over some time period.
    // Data: sum of frequency amplitudes in the frequency band that's looked at. Higher is worse. Flag: Not used. Position of the car on detection.
    OverSpeedLimit = 17,			// Driving over the speed limit
    // Data: The speed of the vehicle. Flag: The speed limit on the current road.  Position of the car on detection.
    NoEntry = 18,					// Drove into a "no entry" zone (i.e. on the wrong direction onto a highway, or on a road forbidden for motorized traffic)
    // Data: Unused. Flag: Unused. Position of the car on detection.
    AggresivePedaling = 19,			// Similar to UnevenAcceleration, but shorter term.
    // Data: sum of frequency amplitudes in the frequency band that's looked at. Flag: Unused. Position of the car on detection.
    WrongSideOfRoad = 20,			// Driving on the wrong side of the centerline (experimental)
    // Data: Unused. Flag: Unused. Position of the car on detection.
    ForgotBlinker = 21,				// Driver forgot to blink in an intersection (or when exiting a roundabout).
    // Data: Unused.
    // flag 1 = Should have blinked to the left
    // flag 2 = Should have blinked to the right
    // Position of the car on detection.
    ForgotRoundaboutLeftBlink = 22, // Driver forgot to blink to the left when entering a roundabout, and taking the left exit.
    // Data: Unused. Flag: Unused. Position of the car on detection.


    InstructorSpeedEvent = 23,            // Data: Unused. Flag: Unused. Position unused.

    InstructorUnevenSpeedEvent = 24,      // Data: Unused. Flag: Unused. Position unused.

    InstructorWrongPlacementEvent = 25,   // Data: Unused. Flag: Unused. Position unused.

    InstructorLaneSelectionEvent = 26,    // Data: Unused. Flag: Unused. Position unused.

    InstructorSafeDistanceEvent = 27,     // Data: Unused. Flag: Unused. Position unused.

    InstructorYeildEvent = 28,            // Data: Unused. Flag: Unused. Position unused.

    InstructorSignRuleEvent = 29,         // Data: Unused. Flag: Unused. Position unused.

    InstructorTrafficRuleEvent = 30,      // Data: Unused. Flag: Unused. Position unused.

    InstructorMissingRecheckEvent = 31,   // Data: Unused. Flag: Unused. Position unused.

    InstructorMissingOverviewEvent = 32,  // Data: Unused. Flag: Unused. Position unused.

    InstructorMarkerEvent = 33,           // Data: Unused. Flag: Unused. Position unused.

    InstructorGoodJobEvent = 34,          // Data: Unused. Flag: Unused. Position unused.

    InstructorIgnoreEvent = 35,           // Data: Unused. Flag: Unused. Position unused.

    InstructorExpectationsEvent = 36,     // Data: Unused. Flag: Unused. Position unused.

    InstructorGetOverviewEvent = 37,      // Data: Unused. Flag: Unused. Position unused.

    InstructorCheckTrafficEvent = 38,     // Data: Unused. Flag: Unused. Position unused.

    InstructorMissingMirrorEvent = 39,    // Data: Unused. Flag: Unused. Position unused.

    InstructorDirectionalLightsEvent = 40,// Data: Unused. Flag: Unused. Position unused.

    InstructorPedalMisuseEvent = 41,      // Data: Unused. Flag: Unused. Position unused.

    InstructorParkBrakeEvent = 42,        // Data: Unused. Flag: Unused. Position unused.

    EmergencyBrakeDetection = 43,

    StalledEngine = 44,					// Stalled engine. Data: Unused. Flag indicates the type of advice that should be made:
    // Flag 1: Should disengage the engine before engaging park brake.
    // Flag 2: Should release the park brake before releasing the clutch.
    // Flag 3: The gear is too high.
    // Flag 4: Should press the clutch before coming to a stop.
    // Flag 5: Should release the brake before releasing the clutch.
    // Flag 6: Should release clutch more slowly.
    // Flag 7: Should release clutch more slowly or give more throttle.
    // Flag 8: Should use more clutch or more throttle.
    // Position of the car on detection.

    TrafficLightEvent = 45,         // Event used to synchronize Traffic Lights.
    // Flags: GameObject ID
    // Data: Trafficlight Directions
    //PositiveZ = 0,
    //NegativeZ = 1,
    //PositiveX = 2,
    //NegativeX = 3,
    //PedestrianX = 4,
    //PedestrianZ = 5,
    //PositiveZLeft = 6,
    //PositiveZRight = 7,
    //NegativeZLeft = 8,
    //NegativeZRight = 9,
    //PositiveXLeft = 10,
    //PositiveXRight = 11,
    //NegativeXLeft = 12,
    //NegativeXRight = 13,
    //All = 14,
    // X: Light Data (0 = Green, 1 = Yellow, 2 = Red).
    // Y: Bool HoldRed (0 = false, 1 = true)
    // Z: Bool Blinking (0 = false, 1 = true)

    RecklessDriving = 46,

    NormalBrakingEvent = 47,

    EmergencyReaction = 48,         // flag = GameObject instance ID
    // data = reaction time
    // x, y, z = car position.
    // If no reaction: Data = -1

    LeftFootBrakeEvent = 49,

    HeadOnCollisionRisk = 50,			// The risk of a head-on collision when moving into the opposite lane
    // Data: The head on collision risk, computed as -(minimum_time_til_collision - time_til_overtake)
    // This can be interpreted like this:
    // risk > 2: No way to make the overtake without colliding
    // 2 > risk > 1: Can theoretically make it, but we'll have less than half of the safety distance we want
    // 1 > risk > 0: Can make it, but not within the safety distance we want
    // 0 > risk > -1: Can make it, but its very tight. When we've passed the overtaken car by the distance we want, we have less than 1 sec til crash.
    // -1 > risk > -2: Can make it, but its tight. When we've passed the overtaken car by the distance we want, we have less than 2 sec til crash.
    // Flag: Unused. Position of the car on detection.

    LaneChangeCarBehindMargin = 51,		// Triggered on a lane change, assesses the distance to the car behind in the lane ego entered.
    // flag 0 = idle, no one behind
    // flag 1 = ok distance
    // flag 2 = medium distance
    // flag 3 = short distance
    // flag 4 = critical short distance
    // Data: Distance in meters. Position of ego when the event happened

    PassingOnTheRight = 52,				// Triggered when ego passes a car on the right.
    // Data: Unused. Flag: Unused. Position of ego on detection.

    Overtake = 53,						// An overtake has happened, meaning a car that was
    // in front of us in our lane, is now behind us in our lane, and the other car has not changed lanes in between.
    // Data: Unused. Flag: Unused. Position of ego on detection.
    SelectTransmissionType = 54,		// Selects transmission type in CarQt
    // flag = index of selected transmission in MainSTore
    // Data and position not used

    UncomfortableDriving = 55,			// Uncomfortable driving detected. Data: A discomfort score,
    // where score >= 4: quite uncomfortable, >= 5: very uncomfortable
    // Flag: Unused. Position of ego on detection.

    RollbackPosition = 56,              // Event used to move ego 30 seconds back n time
    // Data: Unused. Flag: Unused. XYZ: Unused.
    EyetrackerEvent = 57,               // Event used to send EyetrackerData.
    // Flag: ID of data to send.
    //    0 RearviewMirror,
    //    1 LeftMirror,
    //    2 RightMirror,
    //    3 MiddleRoad,
    //    4 LeftRoad,
    //    5 RightRoad,
    //    6 LeftBlindZone,
    //    7 RightBlindZone,
    //    8 Speedometer,
    //    9 none

    StopMarker = 58,                    // Event used to signal a stop marker
    // Position (x,y,z): Position of the StopMarker. Data: Unused. Flag: Unused.
    SessionDataChanged = 59,			// Session data changed/updated. Sent by CarQT once it
    // has received new data from SessionServer and propagated that into shared memory for sending to slaves
    NormalAccelerationEvent = 60,		// An acceleration has been performed
    StartOfLesson = 61,                 // Event used to signal start of a lesson
    // Position (x,y,z): Ego Position. Data: Unused. Flag: Event time (vehicle_data.idx).
    EndOfLesson = 62,                   // Event used to signal end of a lesson
    // Position (x,y,z): Ego Position. Data: Unused. Flag: Event time (vehicle_data.idx).
    EOE
}

impl EventType {
    pub fn from(value: u32) -> Option<EventType> {
        let end = &EventType::EOE;
        let max_value = unsafe { *(end as *const EventType as *const u32) };
        if value >= max_value {
            return None;
        }

        return Some(unsafe { std::mem::transmute(value) });
    }
    pub fn to_u32(&self) -> u32 {
        return unsafe { *(self as *const Self as *const u32) };
    }
}
