#[cfg(windows)] extern crate winapi;

use winapi::um::memoryapi::CreateFileMappingW;
use winapi::um::memoryapi::MapViewOfFile;
use winapi::um::memoryapi::UnmapViewOfFile;
use winapi::um::memoryapi::FlushViewOfFile;
use winapi::um::handleapi::CloseHandle;
use winapi::um::errhandlingapi::GetLastError;
use winapi::um::memoryapi::FILE_MAP_WRITE;
use winapi::um::handleapi::INVALID_HANDLE_VALUE;
use winapi::um::winnt::PAGE_READWRITE;
use winapi::shared::winerror::ERROR_ALREADY_EXISTS;
use winapi::ctypes::c_void;

use widestring::U16CString;
use std::ptr;
use log::*;


pub unsafe fn map_region(region_name: &str, size_in_bytes: u32) -> Result<(*mut c_void, *mut c_void, bool), i32>
{
    // Get Windows wide cstring
    let region = U16CString::from_str(region_name).unwrap();

    let handle = CreateFileMappingW(
        INVALID_HANDLE_VALUE,
        ptr::null_mut(),
        PAGE_READWRITE,
        0,
        size_in_bytes,
        region.as_ptr()
    );

    let error = GetLastError();
    let mut already_exists = false;
    if error == ERROR_ALREADY_EXISTS
    {
        // This is fine
        already_exists = true;
    }

    // If there is another error => problem
    if error > 0 && error != ERROR_ALREADY_EXISTS
    {
        format!("[Shared memory]: Region cannot be created: {}", error);
        return Err(error as i32);
    }

    let view = MapViewOfFile(
        handle,
        FILE_MAP_WRITE,
        0,
        0,
        0
    );

    // If there is an error during view mapping = BAD
    let error = GetLastError();
    if error > 0 && error != ERROR_ALREADY_EXISTS
    {
        error!("[Shared memory]: File view cannot be mapped: {}", error);
        return Err(error as i32);
    }

    return Ok((handle as *mut c_void, view as *mut c_void, already_exists));
}

pub unsafe fn flush_view(view: *mut c_void) -> bool {
    let flush = FlushViewOfFile(view, 0);
    return flush == 0;
}

pub unsafe fn unmap_region(handle: *mut c_void, view: *mut c_void) -> bool {
    let unmap = UnmapViewOfFile(view);
    let close = CloseHandle(handle);
    return unmap == 0 && close == 0;
}