use std::ptr;

use winapi::ctypes::c_void;
use log::*;

use super::memory_interface::*;


pub struct SharedMemory<T>  {
    region_name: String,
    region_size: u32,

    already_exists: bool,
    initialized: bool,

    handle: *mut c_void,
    view: *mut T,
}

unsafe impl<T> Sync for SharedMemory<T> {}
unsafe impl<T> Send for SharedMemory<T> {}

impl<T>  SharedMemory<T>
{
    pub fn new(region_name: &str, region_size: u32) -> SharedMemory<T> {
        SharedMemory {
            region_name: String::from(region_name),
            region_size: region_size,
            handle: ptr::null_mut(),
            view: ptr::null_mut(),
            already_exists: false,
            initialized: false,
        }
    }

    pub fn initialize(&mut self) -> bool {
        unsafe {
            return match map_region(self.region_name.as_str(), self.region_size)
            {
                Ok(tuple) => {
                    info!("[Shared memory]: Region {} initialized!", self.region_name);
                    self.handle = tuple.0;
                    self.view = tuple.1 as *mut T;
                    self.already_exists = tuple.2;
                    self.initialized = true;
                    true
                },
                Err(_error) => {
                    error!("[Shared memory]: Region {} initialization failed!", self.region_name);
                    false
                }
            };
        }
    }

    pub fn is_initialized(&self) -> bool {
        self.initialized
    }

    pub fn already_exists(&self) -> bool {
        self.already_exists
    }

    pub fn zero_data(&mut self) {
        unsafe {
            ptr::write_bytes(self.view, 0, 1);
            self.commit();
        }
    }

    pub fn get_data_mut_ref(&self) -> &mut T {
        unsafe {
            let memory = &mut (*self.view);
            return memory;
        }
    }

    pub fn get_data_ref(&self) -> &T {
        unsafe {
            let memory = &(*self.view);
            return memory;
        }
    }

    pub fn commit(&self) {
        unsafe {
            flush_view(self.view as *mut c_void);
        }
    }

    pub fn release(&mut self) {
        unsafe {
            let _result = unmap_region(self.handle as *mut c_void, self.view as *mut c_void);
        }
        self.handle = ptr::null_mut();
        self.view = ptr::null_mut();

        self.initialized = false;
        self.already_exists = false;
    }
}
