use std::fs::{File};
use std::io::{BufReader, Write};

use threadpool::ThreadPool;
use rodio;

#[derive(Clone)]
pub struct Text2Speech {
    thread_pool: ThreadPool,
}
impl Text2Speech {
    pub fn new() -> Self {
        Text2Speech {
            thread_pool: ThreadPool::new(1),
        }
    }

    pub fn enqueue_for_read(&self, text: &'static str) {
        self.enqueue_for_read_in_language(text, "en");
    }

    pub fn enqueue_for_read_in_language(&self, text: &'static str, language: &'static str) {
        // Execute the fetch and playback
        self.thread_pool.execute(move || {
            // Fetch mp3 file from Google API
            let url = google_translate_tts::url(text, language);
            let response = reqwest::blocking::get(&url).unwrap();

            // Save it a file, hack to trick rodio
            let mut file2write = File::create("temp.mp3").unwrap();
            file2write.write_all(response.bytes().unwrap().as_ref()).unwrap();

            // Load audio from file
            let file = std::fs::File::open("temp.mp3").unwrap();
            let device = rodio::default_output_device().unwrap();
            let buffer = BufReader::new(file);

            // Play audio
            println!("Started playing: {}", text);
            let sink = rodio::play_once(&device, buffer).unwrap();
            sink.sleep_until_end();
        });
    }
}

unsafe impl Sync for Text2Speech {}
unsafe impl Send for Text2Speech {}