use std::env;
use std::fs;
use std::path::Path;

use ontology_parser::{generate_api, parse};

const ONTOLOGY_FILE_PATH: &str = "../ontology.owl";

const RUST_API_FOLDER: &str = "rust_api";
const RUST_API_FILE: &str = "knowledge_graph_api.rs";


fn main() {
    // Parse ontology
    let parsed_ontology = parse(ONTOLOGY_FILE_PATH);

    let current_directory = env::current_dir().unwrap();
    let output_directory = env::var_os("OUT_DIR").unwrap();


    // Rust side of the api
    let rust_api_directory = current_directory.join(RUST_API_FOLDER);
    fs::create_dir_all(&rust_api_directory).unwrap();
    let rust_api_file = Path::new(&output_directory).join(RUST_API_FILE);
    let rust_api_file2 = rust_api_directory.join(RUST_API_FILE);

    // Generated code
    let generated_code = generate_api(parsed_ontology.0.clone(),
                                      parsed_ontology.1.clone());

    fs::write(
        &rust_api_file,
        generated_code.0.clone()
    ).unwrap();

    fs::write(
        &rust_api_file2,
        generated_code.0.clone()
    ).unwrap();

    println!("cargo:rerun-if-changed=build.rs");
}