#[no_mangle]
pub extern fn add_car_node(timestamp: u64, instance_id: usize, car: Car, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Car(car);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_car_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_crosswalk_node(timestamp: u64, instance_id: usize, crosswalk: Crosswalk, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Crosswalk(crosswalk);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_crosswalk_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_dynamicobject_node(timestamp: u64, instance_id: usize, dynamicobject: DynamicObject, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::DynamicObject(dynamicobject);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_dynamicobject_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_ego_node(timestamp: u64, instance_id: usize, ego: Ego, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Ego(ego);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_ego_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_intersection_node(timestamp: u64, instance_id: usize, intersection: Intersection, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Intersection(intersection);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_intersection_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_lane_node(timestamp: u64, instance_id: usize, lane: Lane, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Lane(lane);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_lane_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_noentrysign_node(timestamp: u64, instance_id: usize, noentrysign: NoEntrySign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::NoEntrySign(noentrysign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_noentrysign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_pedestrian_node(timestamp: u64, instance_id: usize, pedestrian: Pedestrian, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Pedestrian(pedestrian);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_pedestrian_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_priorityroadsign_node(timestamp: u64, instance_id: usize, priorityroadsign: PriorityRoadSign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::PriorityRoadSign(priorityroadsign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_priorityroadsign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_road_node(timestamp: u64, instance_id: usize, road: Road, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::Road(road);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_road_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_roadsign_node(timestamp: u64, instance_id: usize, roadsign: RoadSign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::RoadSign(roadsign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_roadsign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_speedlimitsign_node(timestamp: u64, instance_id: usize, speedlimitsign: SpeedLimitSign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::SpeedLimitSign(speedlimitsign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_speedlimitsign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_staticobject_node(timestamp: u64, instance_id: usize, staticobject: StaticObject, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::StaticObject(staticobject);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_staticobject_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_stopsign_node(timestamp: u64, instance_id: usize, stopsign: StopSign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::StopSign(stopsign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_stopsign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_testclass1_node(timestamp: u64, instance_id: usize, testclass1: TestClass1, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::TestClass1(testclass1);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_testclass1_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_testclass2_node(timestamp: u64, instance_id: usize, testclass2: TestClass2, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::TestClass2(testclass2);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_testclass2_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_yieldsign_node(timestamp: u64, instance_id: usize, yieldsign: YieldSign, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let node = KnowledgeGraphNode::YieldSign(yieldsign);
    let type_index = node.type_index();
    graph.add_node(timestamp, instance_id, node);
    return type_index;
}

#[no_mangle]
pub extern fn remove_yieldsign_node(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_node(timestamp, index);
}

#[no_mangle]
pub extern fn add_belongsto_relation(timestamp: Timestamp, instance_id: usize, belongsto: belongsTo, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::belongsTo(belongsto);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_belongsto_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_connectsto_relation(timestamp: Timestamp, instance_id: usize, connectsto: connectsTo, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::connectsTo(connectsto);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_connectsto_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_contains_relation(timestamp: Timestamp, instance_id: usize, contains: contains, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::contains(contains);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_contains_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_has_relation(timestamp: Timestamp, instance_id: usize, has: has, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::has(has);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_has_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isat_relation(timestamp: Timestamp, instance_id: usize, isat: isAt, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isAt(isat);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isat_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isbehind_relation(timestamp: Timestamp, instance_id: usize, isbehind: isBehind, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isBehind(isbehind);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isbehind_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isin_relation(timestamp: Timestamp, instance_id: usize, isin: isIn, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isIn(isin);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isin_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isinfront_relation(timestamp: Timestamp, instance_id: usize, isinfront: isInFront, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isInFront(isinfront);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isinfront_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_ison_relation(timestamp: Timestamp, instance_id: usize, ison: isOn, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isOn(ison);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_ison_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isonleft_relation(timestamp: Timestamp, instance_id: usize, isonleft: isOnLeft, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isOnLeft(isonleft);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isonleft_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_isonright_relation(timestamp: Timestamp, instance_id: usize, isonright: isOnRight, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isOnRight(isonright);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_isonright_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_ispartof_relation(timestamp: Timestamp, instance_id: usize, ispartof: isPartOf, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::isPartOf(ispartof);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_ispartof_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_passesover_relation(timestamp: Timestamp, instance_id: usize, passesover: passesOver, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::passesOver(passesover);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_passesover_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_testrelation_relation(timestamp: Timestamp, instance_id: usize, testrelation: testRelation, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::testRelation(testrelation);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_testrelation_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_walkson_relation(timestamp: Timestamp, instance_id: usize, walkson: walksOn, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::walksOn(walkson);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_walkson_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}

#[no_mangle]
pub extern fn add_yieldstocarin_relation(timestamp: Timestamp, instance_id: usize, yieldstocarin: yieldsToCarIn, from_node_type: usize, from_node_instance_id: usize, to_node_type: usize, to_node_instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let relation = KnowledgeGraphRelation::yieldsToCarIn(yieldstocarin);
    let type_index = relation.type_index();
    let from_node_index = Index::new(from_node_type, from_node_instance_id);
    let to_node_index = Index::new(to_node_type, to_node_instance_id);
    graph.add_relation(timestamp, instance_id, relation, from_node_index, to_node_index);
    return type_index;
}

#[no_mangle]
pub extern fn remove_yieldstocarin_relation(timestamp: u64, type_index: usize, instance_id: usize, memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let memory = unsafe { &mut *memory_ptr };
    let graph: &mut KnowledgeGraph = memory.get_data_mut_ref();
    let index = Index::new(type_index, instance_id);
    return graph.remove_relation(timestamp, index);
}