use shared_memory::*;
use knowledge_graph::graph::*;
use knowledge_graph::graph::types::*;

use shared_memory_interface;
use shared_memory_interface::*;

use config::*;


// Shared memory interface
#[no_mangle]
pub extern fn initialize_knowledge_graph() -> *mut SharedMemory<KnowledgeGraph> {
    // Initialize memory
    let memory_ptr = initialize_shared_memory::<KnowledgeGraph>(KNOWLEDGE_GRAPH_SHARED_REGION_NAME,
                                              KNOWLEDGE_GRAPH_SHARED_REGION_SIZE);
    // Reset graph
    let memory = unsafe { &mut *memory_ptr };
    let graph = memory.get_data_mut_ref();
    graph.initialize();
    graph.clear();

    return memory_ptr;
}

#[no_mangle]
pub extern fn release_knowledge_graph(memory_ptr: *mut SharedMemory<KnowledgeGraph>) {
    release_shared_memory::<KnowledgeGraph>(memory_ptr);
}

#[no_mangle]
pub extern fn reset_knowledge_graph(memory_ptr: *mut SharedMemory<KnowledgeGraph>) {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_mut_ref().clear();
}

#[no_mangle]
pub extern fn get_node_count(memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_ref().node_count()
}

#[no_mangle]
pub extern fn get_relation_count(memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> usize {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_ref().relation_count()
}

#[no_mangle]
pub extern fn get_last_update(memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> u64 {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_ref().last_update()
}

#[no_mangle]
pub extern fn is_empty(memory_ptr: *mut SharedMemory<KnowledgeGraph>) -> bool {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_ref().is_empty()
}


#[no_mangle]
pub extern fn start_transaction(memory_ptr: *mut SharedMemory<KnowledgeGraph>) {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_mut_ref().start_transaction();
}

#[no_mangle]
pub extern fn commit_transaction(memory_ptr: *mut SharedMemory<KnowledgeGraph>) {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_mut_ref().commit_transaction();
}

#[no_mangle]
pub extern fn set_autocommit(autocommit: bool, memory_ptr: *mut SharedMemory<KnowledgeGraph>) {
    let graph = unsafe { &mut *memory_ptr };
    graph.get_data_mut_ref().set_autocommit(autocommit);
}

// Include the generated knowledge graph api
include!(concat!(env!("OUT_DIR"), "/knowledge_graph_api.rs"));
