﻿using UnityEditor;
using UnityEngine;
using System;

public static class IdPool {
    public static int idCounter = 0;
    public static int Id
    {
        get
        {
            return idCounter;
        }
    }
    public static int GenerateNewId()
    {
        idCounter++;
        return idCounter;
    }
}

// Place this file inside Assets/Editor
[CustomPropertyDrawer(typeof(UniqueIdentifierAttribute))]
public class UniqueIdentifierDrawer : PropertyDrawer
{
    public override void OnGUI(Rect position, SerializedProperty prop, GUIContent label)
    {
        // Generate a unique ID
        if (prop.intValue == -1)
        {
            prop.intValue = IdPool.GenerateNewId();
        }

        // Place a label so it can't be edited by accident
        Rect textFieldPosition = position;
        textFieldPosition.height = 16;
        DrawLabelField(textFieldPosition, prop, label);
    }

    void DrawLabelField(Rect position, SerializedProperty prop, GUIContent label)
    {
        EditorGUI.LabelField(position, label, new GUIContent(prop.intValue.ToString()));
    }
}
