﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;

public class CarRelationUtils
{
    // Threshold is in 0-1 range
    // 1 means we consider all vectors from exactly the same direction to nearly perpendicular
    // 0 means only the vectors that are facing exactly the same direction
    public static bool FacingSameDirection(Vector3 first, Vector3 second, float diffThreshold)
    {
        var dot = Vector3.Dot(first, second);
        var cross = Vector3.Cross(first, second);

        return dot > 0 && cross.magnitude < diffThreshold;
    }

    // Threshold is in 0-1 range
    // 1 means we consider all vectors from exactly the same direction to nearly perpendicular
    // 0 means only the vectors that are facing exactly the same direction
    public static bool FacingOppositeDirection(Vector3 first, Vector3 second, float diffThreshold)
    {
        var dot = Vector3.Dot(first, second);
        var cross = Vector3.Cross(first, second);

        return dot < 0 && cross.magnitude < diffThreshold;
    }

    public enum CarPosition
    {
        Front, FrontRight, Right, BehindRight, Behind, BehindLeft, Left, FrontLeft, Overlapping
    }

    // NOTE: y coordinate is not used!
    public static CarPosition CarPositionRelativeToEgo(Vector3 egoPosition, Vector3 egoDirection, Vector3 carPosition, float egoWidthOffset, float egoLengthOffset)
    {
        // Zero the y coordinate, just in case
        egoPosition.y = 0;
        egoDirection.y = 0;
        carPosition.y = 0;

        
        // Shift the origin of car coords to ego position
        var carLocalCoords = carPosition - egoPosition;

        // Rotate to align ego direction to negative z axis
        Quaternion rotation = Quaternion.identity;
        rotation.SetFromToRotation(egoDirection, new Vector3(0,0,-1));
        Vector3 carLocalCoordsRotated = rotation * carLocalCoords;


        // Set ego boundaries
        var xLeft = -egoWidthOffset;
        var xRight = egoWidthOffset;
        var zFront = egoLengthOffset;
        var zBack = -egoLengthOffset;

        // Determine car position
        // Behind
        if (carLocalCoordsRotated.z < zBack && carLocalCoordsRotated.x > xLeft && carLocalCoordsRotated.x < xRight)
        {
            return CarPosition.Behind;
        }
        else if (carLocalCoordsRotated.z < zBack && carLocalCoordsRotated.x < xLeft)
        {
            return CarPosition.BehindLeft;
        }
        else if (carLocalCoordsRotated.z < zBack && carLocalCoordsRotated.x > xRight)
        {
            return CarPosition.BehindRight;
        }
        // Right or left
        else if (carLocalCoordsRotated.z > zBack && carLocalCoordsRotated.z < zFront && carLocalCoordsRotated.x < xLeft)
        {
            return CarPosition.Left;
        }
        else if (carLocalCoordsRotated.z > zBack && carLocalCoordsRotated.z < zFront && carLocalCoordsRotated.x > xRight)
        {
            return CarPosition.Right;
        }
        // In front
        else if (carLocalCoordsRotated.z > zFront && carLocalCoordsRotated.x > xLeft && carLocalCoordsRotated.x < xRight)
        {
            return CarPosition.Front;
        }
        else if (carLocalCoordsRotated.z > zFront && carLocalCoordsRotated.x < xLeft)
        {
            return CarPosition.FrontLeft;
        }
        else if (carLocalCoordsRotated.z > zFront && carLocalCoordsRotated.x > xRight)
        {
            return CarPosition.FrontRight;
        }


        return CarPosition.Overlapping;
    }
}
